@@ -26,6 +26,8 @@
 
 #include "download.h"
 
+G_BEGIN_DECLS
+
 void curl_download_init ();
 void curl_download_uninit ();
 int curl_download_get_url ( const char *hostname, const char *uri, FILE *f, DownloadMapOptions *options, gboolean ftp, DownloadFileOptions *file_options, void *handle );
@@ -33,4 +35,6 @@ int curl_download_uri ( const char *uri, FILE *f, DownloadMapOptions *options, D
 void * curl_download_handle_init ();
 void curl_download_handle_cleanup ( void * handle );
 
+G_END_DECLS
+
 #endif
