@@ -31,7 +31,7 @@ G_BEGIN_DECLS
 
 void vik_ext_tools_register ( VikExtTool *tool );
 void vik_ext_tools_unregister_all ();
-void vik_ext_tools_add_menu_items ( VikWindow *vwindow, GtkUIManager *uim );
+void vik_ext_tools_add_action_items ( VikWindow *vwindow, GtkUIManager *uim, GtkActionGroup *action_group, guint mid );
 
 G_END_DECLS
 
